//  Common.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/DialogS.h>
#include <Xm/MessageB.h>
#include <Xm/SelectioB.h>
#include "MYudit.h"


#define DIALOG_NOTYET_POPPED_UP		0
#define DIALOG_POPPED_UP 		1
#define DIALOG_ALREADY_POPPED_DOWN 	2

//
// Wait for confirmation from "Motif Tools" by David Flanagan
// ISBN 1-56592-044-9
//
void 
MWait4Confirm (Widget w)
{
    Widget diashell, topshell;
    Window diawindow, topwindow;
    Display *dpy;
    XWindowAttributes xwa;
    XEvent event;
    XtAppContext cxt;
    int continue_state = DIALOG_NOTYET_POPPED_UP;
    //extern void XmUpdateDisplay() ;
    
    for (diashell = w; !XtIsShell(diashell); diashell = XtParent(diashell)) ;

    for (topshell = diashell;
	 !XtIsTopLevelShell(topshell);
	 topshell = XtParent(topshell));
    
    if (XtIsRealized(diashell)) {
	dpy = XtDisplay(diashell);
	diawindow = XtWindow(diashell);

	if(XtIsRealized(topshell))
		topwindow = XtWindow(topshell);
	else
		topwindow = 0;

	cxt = XtWidgetToApplicationContext(diashell);
	
	/* Wait for the dialog to be mapped. */
	while(continue_state != DIALOG_ALREADY_POPPED_DOWN) {

	    /* If mapped the first time, then ... */

	    if((continue_state == DIALOG_NOTYET_POPPED_UP)&&
	       (XGetWindowAttributes(dpy, diawindow, &xwa)&& 
	       xwa.map_state == IsViewable))
	    {
		continue_state = DIALOG_POPPED_UP;
	    }

	    /* If unmapped after the being mapped ...*/

	    if((continue_state == DIALOG_POPPED_UP)&&
	       XGetWindowAttributes(dpy, diawindow, &xwa)&&
	       xwa.map_state != IsViewable)
	    {
		continue_state = DIALOG_ALREADY_POPPED_DOWN;

	    }
		
    /*
     * unless the primary is (or becomes) unviewable or unmapped, it's
     * probably iconified, and nothing will happen.
     * We make an exception to this case when topwindow==diawindow
     */
	    if((topwindow != 0) && (topwindow != diawindow) &&
		(XGetWindowAttributes(dpy, topwindow, &xwa),
		 xwa.map_state != IsViewable))
		break;

	    /*
	     * At this stage, we are guaranteed there will be
	     * an event of some kind.  Beware; we are presumably
	     * in a callback, so this can recurse.
	     */
	    XtAppNextEvent(cxt, &event);
	    XtDispatchEvent(&event);
	}
    }
    
    /* The next XSync() will get an expose event if the dialog 
     * was unmapped. 
     */

    XmUpdateDisplay(topshell);
}

void
MGetValueArray (Widget *w,  char **value, int count)
{
	int		i;
	char		*vle;
	char		type;
	Widget		pushbutton;
	XmString	xmstr;


	for (i=0; i<count; i++)
	{
		if (XmIsTextField (w[i]))
		{
			vle = XmTextGetString (w[i]);
			if (vle == NULL) continue;
			strcpy (value[i], vle);
			XtFree (vle);
		}
		else if (XmIsToggleButton (w[i]))
		{
			if (XmToggleButtonGetState (w[i]))
			{
				*value[i]=1;
			}
			else
			{
				*value[i]=0;
			}
		}
		/* This must be an option menu */
		else if (XmIsRowColumn (w[i]))
		{
			XtVaGetValues (w[i], 
				XmNrowColumnType, &type, 
				XmNmenuHistory, &pushbutton,
				NULL);

			if (type != XmMENU_OPTION)
			{
				continue;
			}
			XtVaGetValues (pushbutton, XmNlabelString, &xmstr, NULL);
			if (XmStringGetLtoR (xmstr, XmSTRING_DEFAULT_CHARSET,
				&vle))
			{
				strcpy (value[i], vle);
				XtFree (vle);
			}
			XmStringFree (xmstr);
		}
	}
}

void
MSetValueArray (Widget *w,  char **value, int count)
{
	int		i, j;
	int		nchildren;
	Widget		*children;
	XmString	xmstr;
	XmString	labelStr;
	char		type;
	Widget		pane;

	for (i=0; i<count; i++)
	{
		if (XmIsTextField (w[i]))
		{
			XmTextSetString (w[i], value[i]);
		}
		else if (XmIsToggleButton (w[i]))
		{
			XmToggleButtonSetState (w[i], (int) *value[i], (int) True);
		}
		/* This must be an option menu */
		else if (XmIsRowColumn (w[i]))
		{
			XtVaGetValues (w[i], 
				XmNrowColumnType, &type, 
				XmNsubMenuId, &pane,
				NULL);

			if (type != XmMENU_OPTION)
			{
				continue;
			}

			XtVaGetValues (pane, XmNchildren, &children,
				XmNnumChildren, &nchildren, NULL);

			xmstr = XmStringCreateSimple (value[i]);
			for (j=0; j<nchildren; j++)
			{
				if (!XmIsPushButton (children[j])) continue;
				XtVaGetValues (children[j], 
					XmNlabelString, &labelStr, NULL);
				if (XmStringCompare (xmstr, labelStr))
				{
					XtVaSetValues (w[i],
						XmNmenuHistory, children[j],
						NULL);
					XmStringFree (labelStr);
					break;
				}
				XmStringFree (labelStr);
			}
			if (j>=nchildren)
			{
				/* set it to first item */
				XtVaSetValues (w[i],
					XmNmenuHistory, children[0],
					NULL);
			}
			XmStringFree (xmstr);
		}
	}
}
