//  MainWidget.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MenuShell.h>
#include <Xm/Text.h>
#include <Xm/ScrolledW.h>
#include <Xm/Label.h>
#include <Xm/PanedW.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#include <Xm/Form.h>
#include <X11/Shell.h>
#include <X11/Xatom.h>
#include <Xm/Protocols.h>
#include <Xm/AtomMgr.h>
#include <iostream.h>
#include "MainWidget.h"
#include <UCommon.h>
#include <Translate.h>
#include "MYudit.h"
#include <stdlib.h>
#include <strings.h>
#include <UList.h>
#include <UFontFreeType.h>

static UList<MainWidget*>	mainWidgetList;

int
MainWidget::getInstances ()
{
	return mainWidgetList.getSize();
}

MainWidget*
MainWidget::getFirst ()
{
	return mainWidgetList.getFirst(this);
}

void MDestroyMain (Widget w, MainWidget* mainWidget, XtPointer p);

MainWidgetAction::MainWidgetAction (const Config& configIn) : config (configIn)
{
	mainForm=0;
}

MainWidgetAction::~MainWidgetAction()
{
	if (mainForm) XtDestroyWidget (mainForm);
}

void
MainWidgetAction::clickedOn (const MenuBarAction::MenuId menuIn, 
	const char* stringIn, const MenuBarAction::MenuState onOff)
{
}

MainWidget::MainWidget (
	const char* nameIn, 
	Widget parentIn,
	const Config& configIn, 
	const char* fileNameIn,
	const char* encodingIn,
	const UWidget::UEditable editableIn) 
	: MainWidgetAction (configIn)
{
	Atom		atom;
	const char*	inputVle;
	const char*	backg;
	AString		fullFileName;
	XtPointer	cludge;
	void*		vcludge;
	
	isEditable = editableIn;
	parent = parentIn;
	name = nameIn;
	mainForm = 0;
	menuBar = 0;
	editForm = 0;
	editArea = 0;
	mainWidget = 0;
	saveDialog = 0;
	loadDialog = 0;
	printDialog = 0;
	findDialog = 0;

	if (encodingIn!=0)
	{
		config.setChecked (Config::ENCODING, encodingIn);
	}
	mainWidgetList.addItem (this, 0);

	mainWidget = XtVaCreatePopupShell (name, applicationShellWidgetClass,
		parent, 0, XmNwidth, 400, XmNheight, 300, 0);

	mainForm = XtVaCreateManagedWidget ("MainForm", xmFormWidgetClass,
		mainWidget, XmNresizePolicy, XmRESIZE_NONE, 0);

	menuBar = new MenuBar (this, "MenuBar");

	backg = (const char*) config.getChecked (Config::WINDOW_BACKGROUND);
	editForm = XtVaCreateManagedWidget ("EditForm",
		xmFormWidgetClass, 	mainForm, 
		XmNresizePolicy, 	XmRESIZE_NONE,
		XmNtopWidget,		menuBar->menuBar,
		XmNleftAttachment, 	XmATTACH_FORM,
		XmNrightAttachment, 	XmATTACH_FORM,
		XmNbottomAttachment, 	XmATTACH_FORM,
		XmNtopAttachment, 	XmATTACH_WIDGET,
		XtVaTypedArg,		XmNbackground, 
		XmRString, backg, strlen (backg) +1,
		0);

	editArea = new MUWidget ("Yudit", editForm, UWidget::EDITABLE); 
	
	XtAddCallback( mainWidget, XmNdestroyCallback,
		(XtCallbackProc) MDestroyMain, this);

	atom = XmInternAtom (XtDisplay(mainWidget), "WM_DELETE_WINDOW", False);
	vcludge = this;
	cludge = vcludge;
	XmAddWMProtocolCallback (mainWidget, atom, 
		(XtCallbackProc) MDestroyMain, cludge);
	XtVaSetValues (mainWidget, XmNdeleteResponse, XmDO_NOTHING, 0);
	XtPopup (mainWidget, XtGrabNone);

	editArea->setupWidget ((const char*)config.getChecked(Config::FONTNAME),
		atoi((const char*)config.getChecked(Config::FONTSIZE)),
		(const char*)config.getChecked(Config::FONTWEIGHT),
		(const char*)config.getChecked(Config::FONTSLANT),
		(const char*)config.getChecked(Config::FONTSPACING),
		(const char*)config.getChecked(Config::FONT_ADD_STYLE),
		(const char*)config.getChecked(Config::FONT_AVG_WIDTH));

	

	if (editArea->setConversion((const char*)config.getChecked(Config::ENCODING)) == UString::ERROR)
	{
		cerr << "myudit: can not find converter for encoding '" << "\n";
	}
	inputVle = (const char*)config.getChecked(Config::INPUT);
	if (strcasecmp (inputVle, "Straight") ==0
		|| strcasecmp (inputVle, "None")==0)
	{
			editArea->setInput (0);
	}
	else if (editArea->setInput(inputVle) == UWidget::ERROR)
	{
		cerr << "myudit: can not set input '" <<
			(const char*)config.getChecked(Config::INPUT) << "'\n";
	}
	inputVle = (const char*)config.getChecked(Config::INPUT);
	if (inputVle!=0)
	{
		// set up xinput here when it is supported
	}
	
	editArea->setTabSize (
		atoi((const char*) config.getChecked (Config::TABSIZE)));
	editArea->setBackground (
		(const char*) config.getChecked (Config::WINDOW_BACKGROUND));
	editArea->setForeground(
		(const char*) config.getChecked (Config::WINDOW_FOREGROUND));
	editArea->setCursorColor(
		(const char*) config.getChecked (Config::CURSOR_COLOR));
	setXInput ((const char*) config.getChecked (Config::XINPUT));
	encoding = (const char*) 0;
	if (fileNameIn!=0)
	{
		fullFileName = "file:";
		fullFileName+=fileNameIn;
		isEditable = UWidget::EDITABLE;
		editArea->uSetEditable (UWidget::EDITABLE);
		loadFile ((const char*) fullFileName);
		XtVaSetValues (mainWidget, XmNtitle, fileName.isNull()
			? MYuditTitle() : (const char*)fileName, 0);
	}
	isEditable = editableIn;
	editArea->uSetEditable (editableIn);
	
}

MainWidget::~MainWidget ()
{
	XtPopdown (mainWidget);
	if (menuBar) delete menuBar;
	if (saveDialog) delete saveDialog;
	if (loadDialog) delete loadDialog;
	if (printDialog) delete printDialog;
	if (findDialog) delete findDialog;
	if (editArea != 0) delete editArea;
	if (editForm) XtDestroyWidget (editForm);
	
	XtRemoveCallback( mainWidget, XmNdestroyCallback,
		(XtCallbackProc) MDestroyMain, this);
	XtDestroyWidget (mainWidget);
	mainWidgetList.deleteItem (this);
	if (mainWidgetList.getSize() == 0)
	{
		exit (0);
	}
}

void 
MDestroyMain (Widget w, MainWidget* mainWidget, XtPointer p)
{
	if (mainWidget->editArea->uIsModified())
	{
		if (mainWidget->ask (Translate ("Buffer has been modified.\nDo you really want to quit?")) == MainWidget::NO)
		{
				return;
		}
	}
	else if (mainWidget->getInstances() == 1)
	{
		if (mainWidget->ask (Translate ("Do you really want to exit?")) == MainWidget::NO)
		{
			return;
		}
	}
	delete mainWidget;
}

MenuBar::~MenuBar ()
{
	int 	i;

	for (i=0; i<UMENUMAX; i++)
	{
		if (menu[i]!=0) delete menu[i];
	}
	if (menuWindow!=0) delete menuWindow;
	if (menuBar!=0) XtDestroyWidget (menuBar);
}

MenuBar::MenuBar (MainWidgetAction* mainWidgetIn, const char* name) 
	: MenuBarAction (name)
{
	mainWidget = mainWidgetIn;
	menuWindow = 0;

	// Menu Bar
	menuBar = XtVaCreateManagedWidget (name, xmRowColumnWidgetClass,
		mainWidgetIn->mainForm, 
		XmNrowColumnType, XmMENU_BAR, 
		XmNx, 0, XmNy, 0, 
		XmNmenuAccelerator, "<KeyUp>F10",
		XmNrightAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNtopOffset, 0, 0);

	memset (menu, 0, UMENUMAX * sizeof (MenuRC*));
	menu[UFILE] = new MenuRC (menuBar, this, UFILE, NORMAL,
			"&File", "FilePullDown");
	CHECKNULL (menu[UFILE]);

	//
	// Translate should get these...
	//
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Open...");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Append...");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Re-Load");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&New Window");
	menu[UFILE]->addItem (MenuRC::SEPARATOR, "Separator");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Save");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Save As...");
	menu[UFILE]->addItem (MenuRC::SEPARATOR, "Separator");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Config");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Print");
	menu[UFILE]->addItem (MenuRC::SEPARATOR, "Separator");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Close");
	menu[UFILE]->addItem (MenuRC::BUTTON, "&Exit All");

	// Encoding
	menu[UENCODING] = new MenuRC (menuBar, this, UENCODING, RADIO,
			"&Encoding", "FilePullDown");

	CHECKNULL (menu[UENCODING]);
	menu[UENCODING]->addButtons (mainWidget->getConfig().getCheckList(Config::ENCODING)); 
	clickOn (UENCODING, (const char*) mainWidget->getConfig().getChecked (Config::ENCODING));

	// Input
	menu[UINPUT] = new MenuRC (menuBar, this, UINPUT, RADIO,
		"&Input", "Input");
	CHECKNULL (menu[UINPUT]);
	menu[UINPUT]->addButtons (mainWidget->getConfig().getCheckList(Config::INPUT)); 
	clickOn (UINPUT, (const char*) mainWidget->getConfig().getChecked (Config::INPUT));

	// XInput
	menu[UINPUT]->addItem (MenuRC::SEPARATOR, "Separator");
	menu[UXINPUT] = new MenuRC (menu[UINPUT]->manager, this, UXINPUT, RADIO,
		"&XInput", "XInput");
	CHECKNULL (menu[UXINPUT]);
	menu[UXINPUT]->addButtons (mainWidget->getConfig().getCheckList(Config::XINPUT)); 
	if (!mainWidget->getConfig().getChecked (Config::XINPUT).isNull())
	{
		clickOn (UXINPUT, (const char*) mainWidget->getConfig().getChecked (Config::XINPUT));
	}

	// Font
	menu[UFONTNAME] = new MenuRC (menuBar, this, UFONTNAME, RADIO,
		"&Font", "Font");
	CHECKNULL (menu[UFONTNAME]);
	menu[UFONTNAME]->addButtons (mainWidget->getConfig().getCheckList(Config::FONTNAME)); 
	clickOn (UFONTNAME, (const char*) mainWidget->getConfig().getChecked (Config::FONTNAME));
	menu[UFONTNAME]->addItem (MenuRC::SEPARATOR, "Separator");

	// Font Size
	menu[UFONTSIZE] = new MenuRC (menu[UFONTNAME]->manager, this, UFONTSIZE, RADIO,
		"&Size", "Size");
	CHECKNULL (menu[UFONTSIZE]);
	menu[UFONTSIZE]->addButtons (mainWidget->getConfig().getCheckList(Config::FONTSIZE)); 
	clickOn (UFONTSIZE, (const char*) mainWidget->getConfig().getChecked (Config::FONTSIZE));
	
	// Font Weight
	menu[UFONTWEIGHT] = new MenuRC (menu[UFONTNAME]->manager, this, UFONTWEIGHT, RADIO,
		"&Weight", "Weight");
	CHECKNULL (menu[UFONTWEIGHT]);
	menu[UFONTWEIGHT]->addButtons (mainWidget->getConfig().getCheckList(Config::FONTWEIGHT)); 
	clickOn (UFONTWEIGHT, (const char*) mainWidget->getConfig().getChecked (Config::FONTWEIGHT));

	// Font Slant
	menu[UFONTSLANT] = new MenuRC (menu[UFONTNAME]->manager, this, UFONTSLANT, RADIO,
		"S&lant", "Slant");
	CHECKNULL (menu[UFONTSLANT]);
	menu[UFONTSLANT]->addButtons (mainWidget->getConfig().getCheckList(Config::FONTSLANT)); 
	clickOn (UFONTSLANT, (const char*) mainWidget->getConfig().getChecked (Config::FONTSLANT));

	// Font Spacing
	menu[UFONTSPACING] = new MenuRC (menu[UFONTNAME]->manager, this, UFONTSPACING, RADIO,
		"S&pacing", "Spacing");
	CHECKNULL (menu[UFONTSPACING]);
	menu[UFONTSPACING]->addButtons (mainWidget->getConfig().getCheckList(Config::FONTSPACING)); 
	clickOn (UFONTSPACING, (const char*) mainWidget->getConfig().getChecked (Config::FONTSPACING));

	// Add.Style
	menu[UFONT_ADD_STYLE] = new MenuRC (menu[UFONTNAME]->manager, this, UFONT_ADD_STYLE, RADIO,
		"A&dd.Style", "Add.Style");
	CHECKNULL (menu[UFONT_ADD_STYLE]);
	menu[UFONT_ADD_STYLE]->addButtons (mainWidget->getConfig().getCheckList(Config::FONT_ADD_STYLE)); 
	clickOn (UFONT_ADD_STYLE, (const char*) mainWidget->getConfig().getChecked (Config::FONT_ADD_STYLE));

	// Avg.Width
	menu[UFONT_AVG_WIDTH] = new MenuRC (menu[UFONTNAME]->manager, this, UFONT_AVG_WIDTH, RADIO,
		"Av&g.Width", "Avg.Width");
	CHECKNULL (menu[UFONT_AVG_WIDTH]);
	menu[UFONT_AVG_WIDTH]->addButtons (mainWidget->getConfig().getCheckList(Config::FONT_AVG_WIDTH)); 
	clickOn (UFONT_AVG_WIDTH, (const char*) mainWidget->getConfig().getChecked (Config::FONT_AVG_WIDTH));

	// Window
	menuWindow = new MenuRC (menuBar, this, UMENUMAX, RADIO,
		"&Window", "Window");
	CHECKNULL (menuWindow);
	
	// Tabsize
	menu[UTABSIZE] = new MenuRC (menuWindow->manager, this, 
		UTABSIZE, RADIO, "&Tab Size", "Tab Size");
	CHECKNULL (menu[UTABSIZE]);
	menu[UTABSIZE]->addButtons (
 	 mainWidget->getConfig().getCheckList(Config::TABSIZE)); 
	clickOn (UTABSIZE, (const char*) 
	 mainWidget->getConfig().getChecked (Config::TABSIZE));

	// Background
	menu[UWINDOW_BACKGROUND] = new MenuRC (menuWindow->manager, this, 
		UWINDOW_BACKGROUND, RADIO, "&Background", "Background");
	CHECKNULL (menu[UWINDOW_BACKGROUND]);
	menu[UWINDOW_BACKGROUND]->addButtons (
 	 mainWidget->getConfig().getCheckList(Config::WINDOW_BACKGROUND)); 
	clickOn (UWINDOW_BACKGROUND, (const char*) 
	 mainWidget->getConfig().getChecked (Config::WINDOW_BACKGROUND));

	// Foreground
	menu[UWINDOW_FOREGROUND] = new MenuRC (menuWindow->manager, this, 
		UWINDOW_FOREGROUND, RADIO, "&Foreground", "Foreground");
	CHECKNULL (menu[UWINDOW_FOREGROUND]);
	menu[UWINDOW_FOREGROUND]->addButtons (
 	 mainWidget->getConfig().getCheckList(Config::WINDOW_FOREGROUND)); 
	clickOn (UWINDOW_FOREGROUND, (const char*) 
	 mainWidget->getConfig().getChecked (Config::WINDOW_FOREGROUND));

	// Cursor
	menu[UCURSOR_COLOR] = new MenuRC (menuWindow->manager, this, 
		UCURSOR_COLOR, RADIO, "&Cursor Color", "Cursor Color");
	CHECKNULL (menu[UCURSOR_COLOR]);
	menu[UCURSOR_COLOR]->addButtons (
 	 mainWidget->getConfig().getCheckList(Config::CURSOR_COLOR)); 
	clickOn (UCURSOR_COLOR, (const char*) 
	 mainWidget->getConfig().getChecked (Config::CURSOR_COLOR));

	// Search
	menu[USEARCH] = new MenuRC (menuBar, this, USEARCH, NORMAL,
			"&Search", "SearchPullDown");
	CHECKNULL (menu[USEARCH]);
	menu[USEARCH]->addItem (MenuRC::BUTTON, "&Find/Replace");
	menu[USEARCH]->addItem (MenuRC::BUTTON, "&Go To");

	// Help
	menu[UHELP] = new MenuRC (menuBar, this, UHELP, NORMAL,
			"&Help", "HelpPullDown");
	CHECKNULL (menu[UHELP]);

	//
	// Translate should get these...
	//
	menu[UHELP]->addItem (MenuRC::BUTTON, "&About");
	menu[UHELP]->addItem (MenuRC::SEPARATOR, "Separator");
	menu[UHELP]->addItem (MenuRC::BUTTON, "&General");
	menu[UHELP]->addItem (MenuRC::BUTTON, "&Config");
	menu[UHELP]->addItem (MenuRC::SEPARATOR, "Separator");
	menu[UHELP]->addItem (MenuRC::BUTTON, "&TestPage");
}

void
MenuBar::clickOn (const MenuId menuIn, const char* stringIn, int callBack)
{
	menu[menuIn]->clickOn (stringIn, callBack);
}

void
MenuBar::clickedOn (const MenuId menuIn, const char* stringIn,
	const MenuBarAction::MenuState onOff)
{
	mainWidget->clickedOn (menuIn, stringIn, onOff);
}

//------------------------------------------------------------------------------
// MenuBarAction
//------------------------------------------------------------------------------

MenuBarAction::MenuBarAction (const char* name)
{
}

MenuBarAction::~MenuBarAction ()
{
}

void
MenuBarAction::clickedOn (const MenuId menuIn, const char* stringIn,
	const MenuBarAction::MenuState onOff)
{
}

//------------------------------------------------------------------------------
// MenuRC
//------------------------------------------------------------------------------
MenuRC::MenuRC (Widget			parentWidget,
		MenuBarAction*          parentIn,
		MenuBarAction::MenuId   idIn,
		MenuBarAction::MenuType typeIn,
		const char*             nameTop,
		const char*		namePulldown)
{
	char*		pureOrig;
	char*		menuName;
	char*		menuMneno;

	menuSize = 0;
	menuArraySize = 0;
	menuItems = 0;
	type = typeIn;
	id = idIn;
	parent=parentIn;

	translate (nameTop, &pureOrig, &menuName, &menuMneno);

	// Popup Shell
	menuPopup = XtVaCreatePopupShell (namePulldown, 
		xmMenuShellWidgetClass, parentWidget, 
		XmNwidth, 1, XmNheight, 1, 
		XmNallowShellResize, TRUE,
		XmNoverrideRedirect, TRUE, 0);
	
	// Pulldown Menu - this will be the parent of all menu items
	manager = XtVaCreateWidget(namePulldown,
		xmRowColumnWidgetClass, menuPopup, 
		XmNrowColumnType, XmMENU_PULLDOWN, 
		XmNradioBehavior, (typeIn==MenuBarAction::RADIO) ? True : False,
		0);

	// Crreate The Button on the menubar
	if (menuMneno[0] != 0)
	{
		menuButton = XtVaCreateManagedWidget (pureOrig,
			xmCascadeButtonWidgetClass,
			parentWidget,
			XtVaTypedArg, XmNlabelString, 
			XmRString, menuName, strlen (menuName) +1,
			XmNsubMenuId, manager,
			XtVaTypedArg, XmNmnemonic, 
			XmRString, menuMneno, strlen (menuMneno) +1,
			0);
	}
	else
	{
		menuButton = XtVaCreateManagedWidget (pureOrig,
			xmCascadeButtonWidgetClass,
			parentWidget,
			XtVaTypedArg, XmNlabelString, 
			XmRString, menuName, strlen (menuName) +1,
			XmNsubMenuId, manager,
			0);
	}
	if (pureOrig != 0) delete pureOrig;
	if (menuName != 0) delete menuName;
	if (menuMneno != 0) delete menuMneno;
}

MenuRC::~MenuRC ()
{
	int	i;
	for (i=0; i<menuSize; i++)
	{
		if (menuItems[i] != 0) XtDestroyWidget (menuItems[i]);
	}
	if (menuItems!=0) delete menuItems;
	if (menuButton != 0) XtDestroyWidget (menuButton);
	if (manager != 0) XtDestroyWidget (manager);
	if (menuPopup != 0) XtDestroyWidget (menuPopup);
}

void
MenuRC::clickOn (const char* stringIn, int callBack)
{

	int 	i;
	int 	j;
	for (i=0; i<menuSize; i++)
	{
		if (strcmp (stringIn, XtName (menuItems[i]))==0)  break;
	}
	if (i==menuSize)
	{
		cerr << "warn: can not find menu item " << stringIn << "\n";
		return;
	}
	
	if (XmIsToggleButton (menuItems[i]))
	{
		// turn off other ones 
		if (type==MenuBarAction::RADIO) for (j=0; j<menuSize; j++)
		{
			if (XmIsToggleButton (menuItems[j]))
			{
				XmToggleButtonSetState (menuItems[j], 
					False, False);
			}
		}
		XmToggleButtonSetState (menuItems[i], True, 
			((callBack==0) ? False : True));
		return;
	}
	cerr << "warn: can not find suitable menu item " << stringIn << "\n";
	return;
}

void
MenuRC::translate (const char* nameIn, char** pureOrig, 
		char** menuName, char** menuMneno)
{
	int		i;
	int		len;
	int		count;
	int		countM;
	const char*	newText;

	len = strlen (nameIn);
	pureOrig[0] = new char [len +1];
	CHECKNULL (pureOrig[0]);
	for (i=0, count=0; i<len; i++)
	{
		if (nameIn[i] == '&')
		{
			continue;
		}
		pureOrig[0][count++] = nameIn[i];
	}
	pureOrig[0][count] = 0;
	newText = Translate (nameIn);
	len = strlen (newText);

	menuName[0] = new char [len +1];
	CHECKNULL (menuName[0]);

	menuMneno[0] = new char [len +1];
	CHECKNULL (menuMneno[0]);

	for (i=0, count=0, countM=0; i<len; i++)
	{
		if (nameIn[i] == '&')
		{
			menuMneno[0][countM++] = newText[i+1];
			continue;
		}
		menuName[0][count++] = newText[i];
	}
	menuMneno[0][countM] = 0;
	menuName[0][count] = 0;
	return;
}

void
MenuRC::addItem (ButtonType buttonTypeIn, const char *nameIn)
{
	char*	pureOrig;
	char*	menuName;
	char*	menuMneno;
	Widget*	newArray;
	int	newSize;
	int	i;

	if (menuSize >= menuArraySize)
	{
		newSize = (menuArraySize==0)?1:menuArraySize<<1;
		newArray = new Widget[newSize];
		CHECKNULL (newArray);
		for (i=0; i<menuSize; i++)
		{
			newArray[i] = menuItems[i];
		}
		if (menuItems) delete menuItems;
		menuItems = newArray;
		menuArraySize = newSize;
	}

	translate (nameIn, &pureOrig, &menuName, &menuMneno);

	if (menuMneno[0] != 0)
	{
		menuItems[menuSize] = XtVaCreateManagedWidget (pureOrig,
			(buttonTypeIn == SEPARATOR) ? xmSeparatorWidgetClass :
				((type == MenuBarAction::NORMAL) 
					? xmPushButtonWidgetClass :
					xmToggleButtonWidgetClass),
			manager, 
			XtVaTypedArg, XmNlabelString, 
			XmRString, menuName, strlen(menuName) + 1,
			XtVaTypedArg, XmNmnemonic, 
			XmRString, menuMneno, strlen (menuMneno) +1, 
			0);
	}
	else
	{
		menuItems[menuSize] = XtVaCreateManagedWidget (pureOrig,
			(buttonTypeIn == SEPARATOR) ? xmSeparatorWidgetClass :
				((type == MenuBarAction::NORMAL) 
					? xmPushButtonWidgetClass :
					xmToggleButtonWidgetClass),
			manager, 
			XtVaTypedArg, XmNlabelString, 
			XmRString, menuName, strlen(menuName) + 1,
			0);
	}

	if (buttonTypeIn==MenuRC::BUTTON)
	{
		XtAddCallback (menuItems[menuSize], 
			(type == MenuBarAction::NORMAL) ? XmNactivateCallback
				: XmNvalueChangedCallback,
			(XtCallbackProc) MButtonCB, this);
		
	}
	menuSize++;

	if (pureOrig != 0) delete pureOrig;
	if (menuName != 0) delete menuName;
	if (menuMneno != 0) delete menuMneno;
}

void
MenuRC::addButtons (const AStringList&  list)
{
	int		i;

	for (i=0; i<list.getSize(); i++)
	{
		addItem (BUTTON, list.at(i));
	}
}

void
MButtonCB (Widget w, MenuRC* menuRC, XmAnyCallbackStruct *cb)
{
	//
	// see if it was released.
	//
	if (XmIsToggleButton (w) && !XmToggleButtonGetState (w))
	{
		menuRC->getParent()->clickedOn (menuRC->getId(), XtName (w),
			MenuBarAction::UOFF);
		return;
	}
	menuRC->getParent()->clickedOn (menuRC->getId(), XtName (w),
			MenuBarAction::UON);
}

//
// Set toggles to their values in a workproc.
// It is not intended to be an init funciton.
// Supported:
//            UENCODING, UINPUT, UXINPUT, UFONTNAME
void
MainWidget::setToggle ()
{
	XtAppAddWorkProc (MYuditGetTopContext(), (XtWorkProc) MSetToggleCB, this);
}

Boolean
MSetToggleCB (MainWidget* mainWidget)
{
	// See if destroyed.
	if (mainWidgetList.getIndex (mainWidget)<0) 
	{
		return True;
	}
	// Go thru config and set pushbuttons.
	mainWidget->menuBar->clickOn (MenuBarAction::UENCODING, 
		(const char*) mainWidget->config.getChecked (Config::ENCODING));
	mainWidget->menuBar->clickOn (MenuBarAction::UINPUT, 
		(const char*) mainWidget->config.getChecked (Config::INPUT));

	if (!mainWidget->config.getChecked (Config::XINPUT).isNull())
	{
		mainWidget->menuBar->clickOn (MenuBarAction::UXINPUT, 
		 (const char*) mainWidget->config.getChecked (Config::XINPUT));
	}
	mainWidget->menuBar->clickOn (MenuBarAction::UFONTNAME, 
		(const char*) mainWidget->config.getChecked (Config::FONTNAME));
	return True;
}

